package zs.persistence;

import java.io.BufferedWriter;
import java.io.IOException;
import java.time.format.DateTimeFormatter;
import java.util.Collection;

import zs.model.Delivery;
import zs.model.FailedDelivery;
import zs.model.Shipment;
import zs.model.SucceededDelivery;

public class MyDeliveriesWriter implements DeliveriesWriter {
	
	@Override
	public void write(BufferedWriter writer, Collection<Shipment> shipments) throws IOException {
		for(Shipment shipment : shipments) {
			for (Delivery delivery : shipment.getDeliveries()){
				writeDelivery(writer, shipment.getTracking(), delivery);
			}
		}
	}

	private void writeDelivery(BufferedWriter writer, String tracking, Delivery delivery) throws IOException {
		if (delivery instanceof FailedDelivery) {
			writeFailedDelivery(writer, tracking, (FailedDelivery) delivery);
			return;
		}
		if (delivery instanceof SucceededDelivery) {
			writeSucceededDelivery(writer, tracking, (SucceededDelivery) delivery);
			return;
		}
				
		throw new IllegalArgumentException("Delivery unknown: " + 
				delivery.getClass().getName());
	}

	private void writeSucceededDelivery(BufferedWriter writer, String tracking, SucceededDelivery delivery) throws IOException {
		writer.write("Succeeded;");
		writer.write(tracking);
		writer.write(";");
		writer.write(delivery.getDateTime().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME));
		writer.write(";");
		writer.write(delivery.getRecipientSign());
		writer.write(";");
		writer.write(delivery.getNotes());
		writer.newLine(); // writer.write("\n");
	}

	private void writeFailedDelivery(BufferedWriter writer, String tracking, FailedDelivery delivery) throws IOException {
		writer.write("Failed;");
		writer.write(tracking);
		writer.write(";");
		writer.write(delivery.getDateTime().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME));
		writer.write(";");
		writer.write(delivery.getNotes());
		writer.newLine(); // writer.write("\n");
	}
}
