package zs.persistence;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import zs.model.Recipient;
import zs.model.Shipment;

public class MyShipmentsReader implements ShipmentsReader {

	@Override
	public List<Shipment> read(BufferedReader reader) throws IOException, BadFileFormatException {
		ArrayList<Shipment> shipments = new ArrayList<Shipment>();
		String line;
		while ((line = reader.readLine()) != null) {
			Shipment shipment = readShipment(line);
			shipments.add(shipment);
		}
		return shipments;
	}

	private Shipment readShipment(String line) throws BadFileFormatException {
		StringTokenizer tokenizer = new StringTokenizer(line);
		
		checkTokenPresent(tokenizer, "Tracking expected");			
		String trackingToken = tokenizer.nextToken();
		
		checkTokenPresent(tokenizer, "Weight expected");
		String weightToken = tokenizer.nextToken();
		float weight;
		try {
			weight = Float.parseFloat(weightToken);
		} catch (Exception e) {
			throw new BadFileFormatException("Weight malformed", e);
		}
		if (weight <= 0)
			throw new BadFileFormatException("Weight cannot be negative");
		
		checkTokenPresent(tokenizer, "Recipient expected");
		String recipientToken = tokenizer.nextToken("").trim();
		Recipient recipient = readRecipient(recipientToken);
		
		return new Shipment(trackingToken, recipient, weight);
	}
	
	private Recipient readRecipient(String recipientToken) throws BadFileFormatException {
		StringTokenizer tokenizer = new StringTokenizer(recipientToken, "@");
		
		checkTokenPresent(tokenizer, "Name expected");
		String nameToken = tokenizer.nextToken();
		
		checkTokenPresent(tokenizer, "Street expected");
		String streetToken = tokenizer.nextToken();
		
		checkTokenPresent(tokenizer, "Zip Code expected");
		String zipCodeToken = tokenizer.nextToken();
		int zipCode;
		try {
			zipCode = Integer.parseInt(zipCodeToken);
		} catch (Exception e) {
			throw new BadFileFormatException("Zip Code malformed");
		}
		
		checkTokenPresent(tokenizer, "City expected");
		String cityToken = tokenizer.nextToken();
		
		return new Recipient(nameToken, streetToken, zipCode, cityToken);
	}

	private void checkTokenPresent(StringTokenizer tokenizer, String errorMessage) throws BadFileFormatException {
		if (!tokenizer.hasMoreTokens())
			throw new BadFileFormatException(errorMessage);
	}

}
